!
module m_sorpaso
  implicit none
  private
  integer, parameter:: iin=kind (1)
  integer, parameter:: idp=kind (1.0d0)
  !
  real (idp), parameter :: tol  = 1.0d-14 
  !
  public iin, idp, tol, sorpaso, flops_task
  !
  contains
  !
  subroutine sorpaso (aa,bb)
    real (idp), dimension (:,:), intent (inout) :: aa
    real (idp), dimension (:,:), intent (inout) :: bb    
    real (idp) :: r, suma
    !
    integer (iin) :: nn, zz
    integer (iin) :: i, j, k, q
    !
    nn = size (aa,1)
    zz = size (bb,2)    
    !
    do  k=1,(nn-1)
      !
      r = 1.0_idp / aa (k,k) 
      do i = (k + 1), nn
      aa (i,k) = r * aa (i,k)
      end do
      !
      do  j=(k+1), nn
        do  i=(k+1), nn
          aa (i,j) = aa (i,j) - aa (i,k) * aa (k,j)
        end do
      end do
      !
    end do
    !
    do  q = 1, zz
      do  i = 2, nn
        suma = 0.0_idp
        do  j = 1, (i-1)
          suma = suma - aa (i,j) * bb (j,q)
        end do  
        bb (i,q) = suma
      end do
    end do
    !
    do  i = nn, 1, -1
      do q = 1, zz
        suma = 0.0_idp
        do j = (i + 1), nn
          suma = suma - aa (i,j) * bb (j,q)
        end do
        bb (i,q) = suma / aa (i,i)
      end do
    end do   
    !
  end subroutine sorpaso
  !
  function flops_task (n, tdelta) result (dflops)
    integer (iin), intent (in) :: n
    real    (idp), intent (in) :: tdelta
    real    (idp) :: zopera, dflops, n_d
    !
    zopera = 0.0_idp
    !
    if (tdelta > tol) then
      n_d = real (n, kind = idp)
      zopera = (2.0_idp / 3.0_idp)*(n_d**3) + 2.0_idp * (n_d**2)
      dflops = zopera / (1.0d6 * tdelta)
    else
      dflops = 0.0_idp
    end if
    !
  end function flops_task
  !    
end module m_sorpaso
!
program test3
  use m_sorpaso, only: iin, idp, tol, sorpaso, flops_task
  implicit none
  !
  real    (idp), dimension (:,:), allocatable :: aa
  real    (idp), dimension (:,:), allocatable :: bb  
  !
  real    (idp) :: tini, tend, tdelta, dflops
  !
  integer (iin) :: nn, zz
  integer (iin) :: i, j, q
  !
  nn = 1000
  zz = 1  
  !
  allocate (aa (1:nn,1:nn))
  allocate (bb (1:nn,1:zz))  
  !
  do  j = 1, nn
    do  i = 1, nn
    aa (i,j) = real ((j - 1) * nn + i, kind=idp)
    end do
    aa (j,j) = aa (j,j) + real (nn, kind=idp)
  enddo
  !
  do  q = 1, zz
    do  i = 1, nn
      bb (j,q) = real ((q-1)*nn+i, kind=idp)
    end do
  end do 
  !
  !Task
  call cpu_time (tini)
  call sorpaso  (aa,bb)
  call cpu_time (tend)  
  !
  tdelta = abs (tend - tini)
  dflops = flops_task (nn, tdelta)
  !
  write (*,*)
  write (*,*)" size;             nn = ", nn
  write (*,*)" elapsed time; tdelta = ", tdelta
  write (*,*)" Mflops (dp) ; Mflops = ", dflops  
  !
  deallocate (aa)
  deallocate (bb)
  !
end program test3
!
